#include <stdio.h>
#include <unistd.h>
#include <ctype.h>
#include <stdlib.h>
#include <time.h>
#include <stdarg.h>

#define MAX_COLS 	80
#define MAX_ROWS 	24

#define ROCK		0
#define PAPER		1
#define SCISSORS	2
#define EXIT		3

struct game_state {
	int plr_rounds;
	int plr_games;
	int plr_curr_rounds;
	int comp_rounds;
	int comp_games;
	int comp_curr_rounds;
};

enum style {
	angle		= '+',
	horizontal 	= '-',
	vertical	= '|',
};

void clear_screen(void);
void init_screen(void);
void underline_on(void);
void underline_off(void);
void move(int row, int col);
void draw_angles(char sym);
void draw_hline(int row, int col, int len, char sym);
void draw_vline(int row, int col, int len, char sym);
void draw_rock(int row, int col);
void draw_paper(int row, int col);
void draw_scissors(int row, int col);
void play(int choise, int fate, struct game_state *state);
void you_win(struct game_state *state);
void you_lose(struct game_state *state);
void message(char *str);
int get_choise(void);
int move_n_draw(char *msg, int row, int col, ...);


int
main()
{
	struct game_state state = {0, 0, 0, 0, 0, 0};
	int choise, fate;

	srandom(time(NULL));
	init_screen();

	for (;;) {

		choise = get_choise();

		if (choise == EXIT) {
			clear_screen();
			return 0;
		}

		fate = random() % 3;
		play(choise, fate, &state);
	}

	return 0;
}

void
clear_screen(void)
{
	printf("\033[%i;%iH\033[1J\033[0;0H", MAX_ROWS, MAX_COLS);
}

void
init_screen(void)
{
	clear_screen();

	/* draw top border */
	draw_hline(0, 1, MAX_COLS - 2, horizontal);

	/* draw bottom border */
	draw_hline(MAX_ROWS, 1, MAX_COLS - 2, horizontal);

	/* draw left and right borders */
	draw_vline(1, 0, MAX_ROWS - 2, vertical);
	draw_vline(1, MAX_COLS, MAX_ROWS - 2, vertical);

	/* draw inner lines */
	draw_hline(MAX_ROWS/2 - 1, 1, MAX_COLS - 2, horizontal);
	draw_hline(MAX_ROWS/2 - 2, 1, MAX_COLS - 2, horizontal);
	draw_hline(MAX_ROWS - 3, 1, MAX_COLS - 2, horizontal);
	draw_vline(1, 14, MAX_ROWS - 4, vertical);

	/* draw angle symbols */
	draw_angles(angle);

	/* printing labels */
	underline_on();
	move_n_draw("Computer", 1, 16);
	underline_off();
	
	move_n_draw("Current rounds wins..0", 2, 20);
	move_n_draw("Total rounds wins....0", 3, 20);
	move_n_draw("Games wins...........0", 4, 20);

	underline_on();
	move_n_draw("Player", MAX_ROWS/2, 16);
	underline_off();

	move_n_draw("Current rounds wins..0", MAX_ROWS/2 + 1, 20);
	move_n_draw("Total rounds wins....0", MAX_ROWS/2 + 2, 20);
	move_n_draw("Games wins...........0", MAX_ROWS/2 + 3, 20);
	move_n_draw("Commands: 1 -Rock, 2 - Paper, 3 - Scissors, 4 -Quit", MAX_ROWS - 4, 20);
}

void
underline_on(void)
{
	printf("\033[4m");
}

void
underline_off(void)
{
	printf("\033[0m");
}

void
move(int row, int col)
{
	printf("\033[%i;%iH", row + 1, col + 1);
}

void
draw_angles(char sym)
{
	move_n_draw(&sym, 0, 0);		/* left top */
	move_n_draw(&sym, 0, MAX_COLS);		/* right top */
	move_n_draw(&sym, MAX_ROWS, 0);		/* left bottom */
	move_n_draw(&sym, MAX_ROWS, MAX_COLS);	/* right bottom */
	move_n_draw(&sym, MAX_ROWS/2 - 2, 14);	/* inner top */
	move_n_draw(&sym, MAX_ROWS/2 -1, 14);	/* inner central */
	move_n_draw(&sym, MAX_ROWS - 3, 14);	/* inner bottom */
}

void
draw_hline(int row, int col, int len, char sym)
{
	move(row, col);

	while (len > 0) {
		printf("%c", sym);
		len--;
	}
}

void
draw_vline(int row, int col, int len, char sym)
{
	move(row, col);

	while (len > 0) {
		printf("%c\033[B\033[D", sym);
		len--;
	}
}

void
draw_rock(int row, int col)
{
		move_n_draw("           ",  row , col);
		move_n_draw(" _ _ _ _   ",  row + 1, col);
		move_n_draw("| | | | |  ",  row + 2, col);
		move_n_draw("| | | | |\\ ", row + 3, col);
		move_n_draw("|_|_|_|_| |",  row + 4, col);
		move_n_draw("|        / ",  row + 5, col);
		move_n_draw(" \\      /  ", row + 6, col);
		move_n_draw("  |    |   ",  row + 7, col);
		move_n_draw("           ",  row + 8, col);
}

void
draw_paper(int row, int col)
{
		move_n_draw("   _ _ _   ",  row , col);
		move_n_draw(" _| | | |  ",  row + 1, col);
		move_n_draw("| | | | |  ",  row + 2, col);
		move_n_draw("| | | | |\\ ", row + 3, col);
		move_n_draw("|       | |",  row + 4, col);
		move_n_draw("|        / ",  row + 5, col);
		move_n_draw(" \\      /  ", row + 6, col);
		move_n_draw("  |    |   ",  row + 7, col);
		move_n_draw("           ",  row + 8, col);
}

void
draw_scissors(int row, int col)
{
		move_n_draw("__       __", 	row , col);
		move_n_draw("\\ \\     / /", 	row + 1, col);
		move_n_draw(" \\ \\   / / ", 	row + 2, col);
		move_n_draw("  \\ \\_/ /  ",	row + 3, col);
		move_n_draw("  /\\   /\\  ",	row + 4, col);
		move_n_draw(" /\\ \\ /_/\\ ", 	row + 5, col);
		move_n_draw(" \\ \\ \\   / ",	row + 6, col);
		move_n_draw("  \\ \\/  /  ", 	row + 7, col);
		move_n_draw("   \\/  /   ",	row + 8, col);
}

void
play(int choise, int fate, struct game_state *state)
{
	int fail;

	/* draw player's choise */
	switch (choise) {
	case ROCK:
		draw_rock(MAX_ROWS/2, 2);
		break;
	case PAPER:
		draw_paper(MAX_ROWS/2, 2);
		break;
	case SCISSORS:
		draw_scissors(MAX_ROWS/2, 2);
		break;
	}

	/* draw computer's choise */
	switch (fate) {
	case ROCK:
		draw_rock(1, 2);	
		break;
	case PAPER:
		draw_paper(1, 2);
		break;
	case SCISSORS:
		draw_scissors(1, 2);
		break;
	}

	/* if choises are same */
	if (choise == fate) {
		message("    Draw!    ");
	} else {
	/* scoring and redrawing */
		fail = (fate + 2) % 3;

		if (choise == fail) {
			you_lose(state);
			message("   You lose! ");
		} else {
			you_win(state);
			message("   You win!  ");
		}
	}

	while (getchar() != '\n') {}
}

void
you_win(struct game_state *state)
{
	state->plr_rounds += 1;
	printf("\033[%i;42H%i", MAX_ROWS/2 + 3, state->plr_rounds);
	state->plr_curr_rounds += 1;
	printf("\033[%i;42H%i", MAX_ROWS/2 + 2, state->plr_curr_rounds);

	if (state->plr_curr_rounds == 3) {
		state->plr_curr_rounds = 0;
		state->comp_curr_rounds = 0;
		state->plr_games += 1;
		printf("\033[%i;42H%i", MAX_ROWS/2 + 4, state->plr_games);
		printf("\033[%i;42H%i", MAX_ROWS/2 + 2, state->plr_curr_rounds);
		printf("\033[3;42H%i", state->comp_curr_rounds);
	}
}

void
you_lose(struct game_state *state)
{
	state->comp_rounds += 1;
	printf("\033[4;42H%i", state->comp_rounds);
	state->comp_curr_rounds += 1;
	printf("\033[3;42H%i", state->comp_curr_rounds);

	if (state->comp_curr_rounds == 3) {
		state->plr_curr_rounds = 0;
		state->comp_curr_rounds = 0;
		state->comp_games += 1;
		printf("\033[5;42H%i", state->comp_games);
		printf("\033[3;42H%i", state->comp_curr_rounds);
		printf("\033[%i;42H%i", MAX_ROWS/2 + 2, state->plr_curr_rounds);
	}
}

void
message(char *str)
{
	printf("\033[%i;1H|", MAX_ROWS - 1);
	printf(str);
	printf("| Press enter to continue ...");
}

int
get_choise(void)
{
	char tmp;
	int res;

	for (;;) {
		printf("\033[%i;17H\033[2K", MAX_ROWS - 1);
		printf("\033[%i;%iH|\033[%i;1H|   Command   | ", MAX_ROWS - 1, MAX_COLS, MAX_ROWS - 1);

		tmp = getchar();

		if (isdigit(tmp)) {
			res = atoi(&tmp);

			if ((0 < res) && (res < 5)) {
				while (getchar() != '\n') {}
				break;
			}
		}
	}

	return --res;
}

/* wrapper above printf, moves cursor to the indicated position and
prints the string corresponding the fmt format string. It recognites
in fmt the simplest patterns of printf:
%c	- char,
%lf	- double,
%f	- float,
%i	- int,
%s	- string. */
int
move_n_draw(char *fmt, int row, int col, ...)
{
	va_list ap;
	double dval;
	int ival;
	char *tmp, *sval;

	if (fmt == NULL)
		return 1;

	move(row, col);
	va_start(ap, col);
	ival = va_arg(ap, int);

	for (tmp = fmt; *tmp != '\0'; tmp++) {

		if (*tmp != '%') {
			putchar(*tmp);
			continue;
		}

		switch (*++tmp) {
			case 's':
				for (sval = va_arg(ap, char *); *sval != '\0'; sval++)
					putchar(*sval);

				break;
			case 'i':
				ival = va_arg(ap, int);
				printf("%i", ival);

				break;
			case 'c':
				sval = va_arg(ap, char *);
				putchar(*sval);

				break;
			case 'f':
				dval = va_arg(ap, double);
				printf("%lf", dval);

				break;
			case 'l':
				if (*++tmp == 'f') {
					dval = va_arg(ap, double);
					printf("%lf", dval);
				} else {
					return 2;
				}

				break;
			default:
				putchar(*tmp);
		}
	}

	va_end(ap);

	return 0;
}
